echo off

rem Script to install jstatd service on Windows
rem It requires srvany.exe found in the Windows Resource Kit
rem The service will run as LocalSytem user by default
rem Usage: service.bat install/remove [service_name] [/user username] [/pass password]

if "%OS%" == "Windows_NT" setlocal

rem Configure these variables to match your environment
set JAVA_HOME=C:\Program Files\Java\jdk1.6.0_26
set POLICY=C:\jstats\jstatd.all.policy
set SVRANY=C:\Program Files\Windows Resource Kits\Tools\srvany.exe
set SERVICE_USER=LocalSystem
set SERVICE_NAME=jstatd
set DESCRIPTION="Jstatd Service"

if not exist "%SVRANY%" goto noResKit
goto gotJdkHome

:noResKit
echo srvany.exe not found. This is required to run jstatd as a Windows service
echo Please install the Windows Reskit if not done already
goto end

:gotJdkHome
if not exist "%JAVA_HOME%\bin\jstatd.exe" goto noJavaHome

:gotPolicy
if not exist "%POLICY%" goto noPolicyFile
goto prereqsOK

:noJavaHome
echo The JAVA_HOME environment variable is not defined correctly
echo This environment variable is needed to run this program
echo NB: JAVA_HOME should point to a JDK not a JRE
goto end

:noPolicyFile
echo The java security policy file was not found
goto end

:prereqsOK
if "x%1x" == "xx" goto displayUsage
set SERVICE_CMD=%1
shift
if "x%1x" == "xx" goto checkServiceCmd

:checkUser
if "x%1x" == "x/userx" goto checkPassword
if "x%1x" == "x--userx" goto checkPassword
set SERVICE_NAME=%1
shift
if "x%1x" == "xx" goto checkServiceCmd
goto checkUser

:checkPassword
shift
set SERVICE_USER=%1
shift
if "x%1x" == "x/passx" goto getPassword
if "x%1x" == "x--passx" goto getPassword
shift
if "x%1x" == "xx" goto checkServiceCmd
goto checkPassword

:getPassword
shift
if "x%1x" == "xx" goto displayUsage
set PASSWORD=%1
shift

:checkServiceCmd
if /i %SERVICE_CMD% == install goto doInstall
if /i %SERVICE_CMD% == remove goto doRemove
if /i %SERVICE_CMD% == uninstall goto doRemove
echo Unknown parameter "%1"

:displayUsage
echo.
echo Usage: service.bat install/remove [service_name] [/user username] [/pass password]
goto end

:doRemove
rem Remove the service
sc delete jstatd
if not errorlevel 1 goto removed
echo Failed removing '%SERVICE_NAME%' service
goto end

:removed
echo The service '%SERVICE_NAME%' has been removed
goto end

:doInstall
rem Install the service
echo Installing the service '%SERVICE_NAME%' ...
sc create jstatd binPath= "%SVRANY%" DisplayName= "%SERVICE_NAME%" obj= ".\%SERVICE_USER%" password= "%PASSWORD%"
sc description jstatd %DESCRIPTION%
reg.exe add "HKLM\SYSTEM\CurrentControlSet\Services\jstatd\Parameters" /t REG_SZ /v AppDirectory /d "%JAVA_HOME%\bin"
reg.exe add "HKLM\SYSTEM\CurrentControlSet\Services\jstatd\Parameters" /t REG_SZ /v Application /d "%JAVA_HOME%\bin\jstatd.exe"
reg.exe add "HKLM\SYSTEM\CurrentControlSet\Services\jstatd\Parameters" /t REG_SZ /v AppParameters /d "-J-Djava.security.policy=%POLICY%"

:installed
echo The service '%SERVICE_NAME%' has been installed.

:end