#!/usr/bin/perl

#---------------------------------------------------------------#
#                                                               #
#       Script: consolidate_rrds.pl				#
#       Author: ed@linuxcrusaders.org                           #
#       Date: 02/07/2011                                        #
#                                                               #
#---------------------------------------------------------------#


use DBI;
use File::Copy;

######### config section #################### 

$database='testcacti';
$host='db';
$port='3306';
$user='testcacti';
$password='testcacti';
$rrapath='/usr/share/cacti/rra';

######### end of config #####################

($hostname,$description,$dsname)=@ARGV;

usage() if $hostname eq "" || $description eq "" || $dsname eq "" || @ARGV !=3;

$newds="$rrapath/$dsname.rrd";

$dsn = "DBI:mysql:database=$database;host=$hostname;port=$port";
my $dbh = DBI->connect("DBI:mysql:database=$database;host=$host",
                         $user, $password,
                         {'RaiseError' => 1});


$sth = $dbh->prepare(qq[select id from host where hostname="$hostname" and description="$description" ]);
$sth->execute();
my $ref = $sth->fetchrow_hashref();
$hostid=$ref->{'id'};

$sth = $dbh->prepare("select data_template_data.id,data_template_data.data_source_path
                FROM (data_local,data_template_data)
                LEFT JOIN data_input
                ON (data_input.id=data_template_data.data_input_id)
                LEFT JOIN data_template
                ON (data_local.data_template_id=data_template.id)
                WHERE data_local.id=data_template_data.local_data_id
                AND data_local.host_id=$hostid");

$sth2 = $dbh->prepare(qq[update data_template_data
		set data_source_path="<path_rra>/$dsname.rrd"
		where id=?]);

$sth->execute();
while (my $ref = $sth->fetchrow_hashref()) {
        $id=$ref->{'id'};
        $dsfile=$ref->{'data_source_path'};
        $dsfile=~s#[^\/]+\/#$rrapath\/#;
	$dsfilelist{$dsfile}=$id;
	($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($dsfile);
	$dslist{$id}=$dsfile;
	$dsfilesize{$id}=$size;
	$fn_len=length $dsfile;
	$mx_len=$fn_len if $fn_len>$mx_len;
}

print "\n";
if(scalar %dsfilelist == 1){
	print "It looks like the datasource ($dsfile) may already be consolidated.\n\n";
	exit;
}else{
	print "Consolidate the following RRDs into one file: $newds?\n\n";
	$dt_len=length 'data_template_data.id';
	$sz_len=length 'size(bytes)';
	printf "%-${dt_len}s   %-${mx_len}s   %-${sz_len}s\n",'data_template_data.id','filename','size(bytes)';
	foreach $id (sort keys %dslist){
		printf "%-${dt_len}s   %-${mx_len}s   %-${sz_len}s\n",$id,$dslist{$id},$dsfilesize{$id}>0?$dsfilesize{$id}:0;
	}
}

print "\nAnswer [y/n]: ";
$reply=<STDIN>;

while($reply!~/(y|n)/i){
	print "\nAnswer [y/n]: ";
	$reply=<STDIN>;
}

if($reply!~/y/i){
	print "Exiting. No database or file changes made.\n\n";
	exit;
}

$first=1;
$totalsize=0;
foreach $id (sort keys %dslist){
	$dsfile=$dslist{$id};
	if(-e $dsfile){
		if($first){
			move $dsfile,$newds;
			$first=0;
		}else{
			$totalsize+=$dsfilesize{$id};
			unlink $dsfile or warn "Could not unlink $dsfile: $!";
		}
	}

	$sth2->execute($id);

}

$sth->finish();
$sth2->finish();

$dbh->disconnect();

if($totalsize>0){
	$totalsizemb=int $totalsize/1024000;
	print "\nCongratulations! You just saved " . $totalsizemb . "MB space.\n";
}

sub usage{
	print "\nUsage:\n\n";
	print "$0 <hostname> <description> <dsname>\n\n";
	print "Example: $0 javahost \"java process\" jstat_java_process\n";
	print "This will delete all but one RRD file associated with the java process running on javahost and rename it to jstat_java_process.rrd\n";
	print "Note that values for hostname and description need to match the device fields as displayed under Console->Devices\n\n";
	exit;
}
