#!/usr/bin/perl

#---------------------------------------------------------------#
#								#
# 	Script: ssh_jstat.pl for jstat_cacti_template		#
# 	Author: ed@linuxcrusaders.org				#
# 	Date: 03/07/2011					#
#								#
#---------------------------------------------------------------#

my $user='autouser';
my $hostname='java';
my $sshcmd="ssh $user\@$hostname";

#  Run 'jps' on the remote host to get a list of java process names and pass one to this script.
#  Set JAVA_HOME below to match that of the process.

my $hostname=$ARGV[0];
my $jproc=$ARGV[1];

if ($jproc eq ""){
	print "usage: $0 <hostname> <java process>\n";
	exit;
}

$os = $^O;

# set JAVA_HOME to point to the same java as the queried process is using
if($os eq 'linux'){
	$JAVA_HOME='/usr/java/latest';
	$ENV{'PATH'} = "$JAVA_HOME/bin:/bin:/usr/bin:/usr/local/bin";
	$pidcmd="$sshcmd $JAVA_HOME/bin/jps | grep $jproc | cut -d\" \" -f1";
	$jstatf="/tmp/$hostname" . "_$jproc.txt";
	$catcmd="cat $jstatf";
	$rmcmd="rm $jstatf";
	
}elsif($os=~/MSWin/){
	$JAVA_HOME='C:\Program Files\Java\jdk1.6.0_26';
	$ENV{'PATH'} = $JAVA_HOME . '\bin;C:\WINDOWS\system32;C:\WINDOWS;C:\strawberry\c\bin;C:\strawberry\perl\site\bin;C:\strawberry\perl\bin;C:\PsTools';
	#$pidcmd="$sshcmd jps | findstr $jproc";
	$pidcmd=qq[$sshcmd PsExec.exe -s "$JAVA_HOME\\bin\\jps" $hostname 2>nul | findstr $jproc];
	$jstatf="C:\\temp\\$hostname" . "_$jproc.txt";
	$catcmd="type $jstatf";
	$rmcmd="del $jstatf";
	
}else{
	print "Sorry, unable to determine operating system. Exiting.";
	exit;
}

my $pid=`$pidcmd`;
chomp $pid;
$pid=~s/\s+\S+//;

if ($pid!~/\d+/){
	print "pid not detected for java process: $jproc\n";
	exit;
}

# cacti will try to run this script for each graph so this is a way to ensure we only call jstat once per poll
my $cat;

if(-e $jstatf){
	($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($jstatf);
	if ((time-$mtime)<60){
		$cat=`$catcmd`;	
		print "$cat\n";
		exit;
	}else{
		`$rmcmd`;
	}
}

my ($Compiled,$Failed,$Invalid,$CompilerTime,$FailedType,$FailedMethod)=run("jstat -compiler $pid\@$hostname");
my ($Loaded,$LoadedBytes,$Unloaded,$UnloadedBytes,$ClassTime)=run("jstat -class $pid\@$hostname");
my ($S0C,$S1C,$S0U,$S1U,$EC,$EU,$OC,$OU,$PC,$PU,$YGC,$YGCT,$FGC,$FGCT,$GCT)=run("jstat -gc $pid\@$hostname"); 
my ($NGCMN,$NGCMX,$NGC,$S0CMX,$S0C,$S1CMX,$S1C,$ECMX,$EC,$YGC,$FGC)=run("jstat -gcnewcapacity $pid\@$hostname");
my ($S0C,$S1C,$S0U,$S1U,$TT,$MTT,$DSS,$EC,$EU,$YGC,$YGCT)=run("jstat -gcnew $pid\@$hostname");
my ($OGCMN,$OGCMX,$OGC,$OC,$YGC,$FGC,$FGCT,$GCT)=run("jstat -gcoldcapacity $pid\@$hostname");
my ($PGCMN,$PGCMX,$PGC,$PC,$YGC,$FGC,$FGCT,$GCT)=run("jstat -gcpermcapacity $pid\@$hostname");
my ($S0,$S1,$E,$O,$P,$YGC,$YGCT,$FGC,$FGCT,$GCT)=run("jstat -gcutil $pid\@$hostname");

open FH,">$jstatf" or die "unable to write to $jstatf: $!\n";
print FH "NGCMN:$NGCMN NGCMX:$NGCMX NGC:$NGC S0CMX:$S0CMX S0C:$S0C S1CMX:$S1CMX S1C:$S1C ECMX:$ECMX EC:$EC YGC:$YGC ";
print FH "OGCMN:$OGCMN OGCMX:$OGCMX OGC:$OGC OC:$OC FGC:$FGC FGCT:$FGCT GCT:$GCT YGCT:$YGCT ";
print FH "PGCMN:$PGCMN PGCMX:$PGCMX PGC:$PGC PC:$PC ";
print FH "EU:$EU OU:$OU PU:$PU ";
print FH "S0U:$S0U S1U:$S1U TT:$TT MTT:$MTT DSS:$DSS ";
print FH "E:$E O:$O P:$P ";
print FH "Loaded:$Loaded LoadedBytes:$LoadedBytes Unloaded:$Unloaded UnloadedBytes:$UnloadedBytes ClassTime:$ClassTime ";
print FH "Compiled:$Compiled Failed:$Failed Invalid:$Invalid CompilerTime:$CompilerTime";
close FH;

$cat=`$catcmd`;	
print "$cat\n";

sub run{
	my $jstatcmd=shift;
	my $str=`$sshcmd $JAVA_HOME/bin/$jstatcmd`;
	chomp $str;
	$str=~s/[^\n]+\n//;
    	$str=~s/^\s+//;
	return split /\s+/,$str;
}
